package ru.dbotthepony.kstarbound.network.packets

import io.netty.buffer.ByteBuf
import io.netty.buffer.ByteBufInputStream
import io.netty.buffer.ByteBufOutputStream
import io.netty.channel.Channel
import io.netty.channel.ChannelHandlerContext
import io.netty.channel.ChannelInboundHandlerAdapter
import ru.dbotthepony.kstarbound.Starbound
import ru.dbotthepony.kstarbound.client.network.ClientConnection
import ru.dbotthepony.kstarbound.network.Connection
import ru.dbotthepony.kstarbound.network.IClientPacket
import ru.dbotthepony.kstarbound.network.IConnectionDetails
import ru.dbotthepony.kstarbound.network.IPacket
import ru.dbotthepony.kstarbound.network.IServerPacket
import ru.dbotthepony.kstarbound.network.readUTF
import ru.dbotthepony.kstarbound.network.writeUTF
import ru.dbotthepony.kstarbound.server.network.ServerConnection
import ru.dbotthepony.kstarbound.io.readUUID
import ru.dbotthepony.kstarbound.io.writeUUID
import java.io.DataInputStream
import java.io.DataOutputStream
import java.util.UUID

data class HelloPacket(
	override val protocolVersion: Int,
	override val engineVersion: String,
	override val username: String,
	override val password: String,
	override val uuid: UUID
) : IPacket, IConnectionDetails {
	constructor(buff: DataInputStream) : this(buff.readInt(), buff.readUTF(), buff.readUTF(), buff.readUTF(), buff.readUUID())

	override fun write(stream: DataOutputStream) {
		stream.writeInt(protocolVersion)
		stream.writeUTF(engineVersion)
		stream.writeUTF(username)
		stream.writeUTF(password)
		stream.writeUUID(uuid)
	}
}

data class DisconnectPacket(val reason: String) : IServerPacket, IClientPacket {
	constructor(buff: DataInputStream) : this(buff.readUTF())

	override fun write(stream: DataOutputStream) {
		stream.writeUTF(reason)
	}

	override fun play(connection: ServerConnection) {

	}

	override fun play(connection: ClientConnection) {

	}
}

class HelloListener(val connection: Connection, private val channel: Channel) : ChannelInboundHandlerAdapter() {
	init {
		channel.pipeline().addFirst(this)
	}

	fun sendHello(uuid: UUID, username: String = "", password: String = ""): HelloListener {
		val buf = channel.config().allocator.buffer()
		buf.writeUTF("KSTARBOUND HELLO")
		HelloPacket(Starbound.PROTOCOL_VERSION, Starbound.ENGINE_VERSION, username, password, uuid).write(DataOutputStream(ByteBufOutputStream(buf)))
		channel.write(buf)
		channel.flush()
		return this
	}

	override fun channelRead(ctx: ChannelHandlerContext, msg: Any) {
		if (msg is ByteBuf) {
			if (msg.readUTF() != "KSTARBOUND HELLO") {
				connection.helloFailed()
			} else {
				connection.helloReceived(HelloPacket(DataInputStream(ByteBufInputStream(msg))))
			}

			msg.release()
		} else {
			connection.helloFailed()
		}

		channel.pipeline().remove(this)
	}
}
